"""
Unit tests for TN-NT Template tags

This module contains unit tests for custom template tags used in the TN-NT Auth Theme.
The tests ensure that the template tags behave as expected under various conditions.
"""

# Django
from django.template import Context, Template
from django.test import TestCase, modify_settings

# AA Templates: Terra Nanotech
from tnnt_templates import __version__


class TestTemplateTags(TestCase):
    """
    Unit test class for verifying the functionality of TN-NT template tags.

    This class contains test cases to ensure that the custom template tags
    used in the TN-NT Auth Theme behave as expected under various conditions.
    """

    def setUp(self):
        """
        Sets up the test context for the test cases.

        This method initializes a Django template context with a predefined
        content string. The context is used in the template rendering tests
        to verify the behavior of custom template tags.

        :return:
        :rtype:
        """

        self.context = Context(
            {"content": "Lorem ipsum dolor sit amet, consectetur adipiscing elit."}
        )

    def test_should_return_content_when_string_starts_with_substring(self):
        """
        Tests that the template renders the content when the string starts with the specified substring.

        This test verifies that the custom `startswith` filter correctly identifies
        when the `content` string begins with the substring "Lorem" and ensures
        that the content is included in the rendered template.

        :return: None
        """

        template_to_render = Template(
            "{% load tnnt_templates %}"
            '{% if content|startswith:"Lorem" %}{{ content }}{% endif %}'
        )
        rendered_template = template_to_render.render(self.context)

        self.assertInHTML(
            "Lorem ipsum dolor sit amet, consectetur adipiscing elit.",
            rendered_template,
        )

    def test_should_not_return_content_when_string_does_not_start_with_substring(self):
        """
        Tests that the template does not render the content when the string does not start with the specified substring.

        This test verifies that the custom `startswith` filter correctly identifies
        when the `content` string does not begin with the substring "ipsum" and ensures
        that the content is excluded from the rendered template.

        :return: None
        """

        template_to_render = Template(
            "{% load tnnt_templates %}"
            '{% if content|startswith:"ipsum" %}{{ content }}{% endif %}'
        )
        rendered_template = template_to_render.render(self.context)

        self.assertNotIn(
            "Lorem ipsum dolor sit amet, consectetur adipiscing elit.",
            rendered_template,
        )

    @modify_settings(INSTALLED_APPS={"append": "aagdpr"})
    def test_should_return_true_when_app_is_installed(self):
        """
        Tests that the template renders 'True' when the specified app is installed.

        This test verifies the behavior of the custom `is_app_installed` template tag
        by appending the app "aagdpr" to the `INSTALLED_APPS` setting. It ensures
        that the template correctly identifies the app as installed and renders 'True'.

        :return: None
        """

        template_to_render = Template(
            "{% load tnnt_templates %}"
            '{% is_app_installed "aagdpr" as is_myapp_installed %}'
            "{% if is_myapp_installed %}True{% else %}False{% endif %}"
        )
        rendered_template = template_to_render.render(Context({"version": __version__}))

        self.assertEqual(rendered_template, "True")

    @modify_settings(INSTALLED_APPS={"remove": "aagdpr"})
    def test_should_return_false_when_app_is_not_installed(self):
        """
        Tests that the template renders 'False' when the specified app is not installed.

        This test verifies the behavior of the custom `is_app_installed` template tag
        by removing the app "aagdpr" from the `INSTALLED_APPS` setting. It ensures
        that the template correctly identifies the app as not installed and renders 'False'.

        :return: None
        """

        template_to_render = Template(
            "{% load tnnt_templates %}"
            '{% is_app_installed "aagdpr" as is_myapp_installed %}'
            "{% if is_myapp_installed %}True{% else %}False{% endif %}"
        )
        rendered_template = template_to_render.render(Context({"version": __version__}))

        self.assertEqual(rendered_template, "False")



"""
Test cases for the functions in tnnt_templates.checks
"""

# Django
from django.conf import settings
from django.test import TestCase, override_settings

# AA Templates: Terra Nanotech
from tnnt_templates.checks import check_mandatory_settings


class TestCheckMandatorySettings(TestCase):
    """
    Test the check_mandatory_settings function
    """

    @override_settings(
        TNNT_TEMPLATE_ENTITY_ID=123456789,
        TNNT_TEMPLATE_ENTITY_NAME="Starfleet Alliance",
    )
    def test_returns_no_errors_when_all_mandatory_settings_are_present(self):
        """
        Test that the check_mandatory_settings function returns no errors when all mandatory settings are present.

        :return:
        :rtype:
        """

        errors = check_mandatory_settings(None)

        self.assertEqual(errors, [])

    @override_settings()
    def test_returns_errors_when_mandatory_settings_are_missing(self):
        """
        Test that the check_mandatory_settings function returns errors when mandatory settings are missing.

        :return:
        :rtype:
        """

        del settings.TNNT_TEMPLATE_ENTITY_ID
        del settings.TNNT_TEMPLATE_ENTITY_NAME

        errors = check_mandatory_settings(None)

        self.assertEqual(len(errors), 2)
        self.assertEqual(errors[0].msg, "Setting 'TNNT_TEMPLATE_ENTITY_ID' is missing")
        self.assertEqual(
            errors[1].msg, "Setting 'TNNT_TEMPLATE_ENTITY_NAME' is missing"
        )

    @override_settings(TNNT_TEMPLATE_ENTITY_NAME="Starfleet Alliance")
    def test_returns_error_when_one_mandatory_setting_is_missing(self):
        """
        Test that the check_mandatory_settings function returns an error when one mandatory setting is missing.

        :return:
        :rtype:
        """

        del settings.TNNT_TEMPLATE_ENTITY_ID

        errors = check_mandatory_settings(None)

        self.assertEqual(len(errors), 1)
        self.assertEqual(errors[0].msg, "Setting 'TNNT_TEMPLATE_ENTITY_ID' is missing")



"""
Test the AppSettings class
"""

# Standard Library
import unittest
from unittest.mock import patch

# AA Templates: Terra Nanotech
from tnnt_templates.app_settings import AppSettings


class TestAppSettings(unittest.TestCase):
    """
    Test the AppSettings class
    """

    @patch("django.apps.apps.is_installed", return_value=True)
    def test_aagdpr_installed_returns_true(self, mock_is_installed):
        """
        Test the aagdpr_installed method when aagdpr is installed

        :param mock_is_installed:
        :type mock_is_installed:
        :return:
        :rtype:
        """

        self.assertTrue(AppSettings.aagdpr_installed())

    @patch("django.apps.apps.is_installed", return_value=False)
    def test_aagdpr_installed_returns_false(self, mock_is_installed):
        """
        Test the aagdpr_installed method when aagdpr is not installed

        :param mock_is_installed:
        :type mock_is_installed:
        :return:
        :rtype:
        """

        self.assertFalse(AppSettings.aagdpr_installed())



"""
Test the context processor for TNNT template settings.
"""

# Standard Library
import unittest
from unittest.mock import patch

# Django
from django.conf import settings

# AA Templates: Terra Nanotech
from tnnt_templates.context_processors import tnnt_settings


class TestContextProcessorTnntSettings(unittest.TestCase):
    """
    Test the tnnt_settings context processor.
    """

    @patch("tnnt_templates.context_processors.AppSettings")
    def test_tnnt_settings_returns_correct_values(self, MockAppSettings):
        """
        Test the tnnt_settings context processor with custom settings

        :param MockAppSettings:
        :type MockAppSettings:
        :return:
        :rtype:
        """

        MockAppSettings.TNNT_TEMPLATE_ENTITY_ID = 98000030
        MockAppSettings.TNNT_TEMPLATE_ENTITY_TYPE = "corporation"
        MockAppSettings.TNNT_TEMPLATE_ENTITY_NAME = "Terra Nanotech"

        settings.REGISTRATION_VERIFY_EMAIL = True

        request = None  # WSGIRequest is not used in the function
        result = tnnt_settings(request)

        self.assertEqual(
            result["TNNT_TEMPLATE_AA_LOGO"], "/static/allianceauth/images/auth-logo.svg"
        )
        self.assertTrue(result["REGISTRATION_VERIFY_EMAIL"])
        self.assertEqual(result["TNNT_TEMPLATE_ENTITY_ID"], 98000030)
        self.assertEqual(result["TNNT_TEMPLATE_ENTITY_TYPE"], "corporation")
        self.assertEqual(result["TNNT_TEMPLATE_ENTITY_NAME"], "Terra Nanotech")

    @patch("tnnt_templates.context_processors.AppSettings")
    def test_tnnt_settings_handles_missing_settings(self, MockAppSettings):
        """
        Test the tnnt_settings context processor with missing settings to return default values

        :param MockAppSettings:
        :type MockAppSettings:
        :return:
        :rtype:
        """

        del settings.REGISTRATION_VERIFY_EMAIL
        del MockAppSettings.TNNT_TEMPLATE_ENTITY_ID
        del MockAppSettings.TNNT_TEMPLATE_ENTITY_TYPE
        del MockAppSettings.TNNT_TEMPLATE_ENTITY_NAME

        request = None  # WSGIRequest is not used in the function
        result = tnnt_settings(request)

        self.assertEqual(
            result["TNNT_TEMPLATE_AA_LOGO"], "/static/allianceauth/images/auth-logo.svg"
        )
        self.assertTrue(result["REGISTRATION_VERIFY_EMAIL"])
        self.assertEqual(result["TNNT_TEMPLATE_ENTITY_ID"], 1)
        self.assertEqual(result["TNNT_TEMPLATE_ENTITY_TYPE"], "alliance")
        self.assertEqual(result["TNNT_TEMPLATE_ENTITY_NAME"], "")



"""
AppConfig for Terra Nanotech theme.
"""

# Django
from django.apps import AppConfig

# AA Templates: Terra Nanotech
from tnnt_templates import __version__


class TerraNanotechThemeConfig(AppConfig):
    """
    AppConfig for Terra Nanotech theme.
    """

    name = "tnnt_templates.theme.terra_nanotech"
    label = "terra_nanotech"
    verbose_name = f"Terra Nanotech Corp Auth Theme v{__version__}"

    def ready(self):
        """
        Ready

        :return:
        :rtype:
        """

        pass  # pylint: disable=unnecessary-pass



"""
TN-NT Templates content processor
"""

# Django
from django.conf import settings
from django.core.handlers.wsgi import WSGIRequest

# AA Templates: Terra Nanotech
from tnnt_templates.app_settings import AppSettings


def tnnt_settings(request: WSGIRequest) -> dict:  # pylint: disable=unused-argument
    """
    Returning a settings dict
    :param request:
    :return:
    """

    # AA logo
    return_value = {
        "TNNT_TEMPLATE_AA_LOGO": "/static/allianceauth/images/auth-logo.svg",
        "REGISTRATION_VERIFY_EMAIL": getattr(
            settings, "REGISTRATION_VERIFY_EMAIL", True
        ),
        "TNNT_TEMPLATE_ENTITY_TYPE": getattr(
            AppSettings, "TNNT_TEMPLATE_ENTITY_TYPE", "alliance"
        ),
        "TNNT_TEMPLATE_ENTITY_ID": getattr(AppSettings, "TNNT_TEMPLATE_ENTITY_ID", 1),
        "TNNT_TEMPLATE_ENTITY_NAME": getattr(
            AppSettings, "TNNT_TEMPLATE_ENTITY_NAME", ""
        ),
    }

    return return_value



"""
Management command to anonymize all email addresses in the database.
"""

# Standard Library
from urllib.parse import urlparse

# Django
from django.conf import settings
from django.core.management.base import BaseCommand

# Alliance Auth
from allianceauth.authentication.models import User


def get_input(text) -> str:
    """
    Wrapped input to enable import

    :param text:
    :type text:
    :return:
    :rtype:
    """

    return input(text)


class Command(BaseCommand):
    """
    Anonymize all email addresses in the database.
    """

    help = "This command will anonymize each user's email address in the database."

    def add_arguments(self, parser):
        """
        Add argument to parser
        :param parser:
        :type parser:
        :return:
        :rtype:
        """

        parser.add_argument(
            "--noinput",
            "--no-input",
            action="store_true",
            help="Do NOT prompt the user for input of any kind.",
        )

    def _anonymize_user_emails(self) -> None:
        """
        Start the anonymization

        :return:
        :rtype:
        """

        self.stdout.write(msg="Anonymizing email addresses …")

        site_fqdn = urlparse(settings.SITE_URL).netloc

        # Get all users and filter those who need anonymization
        users = User.objects.exclude(email="").exclude(email__endswith=f"@{site_fqdn}")

        # Loop through filtered users and anonymize email address
        for user in users:
            self.stdout.write(f"Anonymizing email address for {user.username} …")

            user.email = f"{user.pk}@{site_fqdn}"
            user.save()

        self.stdout.write(msg=self.style.SUCCESS("Anonymization complete!"))

    def handle(self, *args, **options):  # pylint: disable=unused-argument
        """
        Ask before running …

        :param args:
        :type args:
        :param options:
        :type options:
        :return:
        :rtype:
        """

        self.stdout.write(
            msg=self.style.SUCCESS(
                "This will anonymize each user's email address in the database."
            )
        )

        if not options["noinput"]:
            user_input = get_input(text="Are you sure you want to proceed? (yes/no) ")
        else:
            user_input = "yes"

        if user_input.lower() == "yes":
            self.stdout.write(msg="Starting, Please stand by …")
            self._anonymize_user_emails()
        else:
            self.stdout.write(msg=self.style.WARNING("Aborted."))



"""
Test cases for the management command to anonymizes user emails.
"""

# Standard Library
from io import StringIO
from unittest.mock import patch

# Django
from django.test import TestCase, override_settings

# Alliance Auth (External Libs)
from app_utils.testing import create_fake_user

# AA Templates: Terra Nanotech
from tnnt_templates.management.commands.tnnt_anonymize_emails import Command, get_input


class TestManagementCommandAnonymizeEmails(TestCase):
    """
    Test cases for the management command to anonymizes user emails.
    """

    def call_command(self, *args, **kwargs):
        """
        Call the management command to anonymize emails.

        :param args:
        :type args:
        :param kwargs:
        :type kwargs:
        :return:
        :rtype:
        """

        # Django
        from django.core.management import call_command

        call_command(
            "tnnt_anonymize_emails",
            *args,
            stdout=StringIO(),
            stderr=StringIO(),
            **kwargs,
        )

    def setUp(self):
        """
        Set up the test case by creating a fake user with a specific email address.

        :return:
        :rtype:
        """

        self.user = create_fake_user(character_id=2001, character_name="Wesley Crusher")

        self.user.email = "wesley.crusher@enterprise.space"
        self.user.save()

    @override_settings(SITE_URL="https://auth.testserver.net")
    def test_command_anonymize_emails_with_noinput(self):
        """
        Test that the command anonymizes emails when noinput is set

        :return:
        :rtype:
        """

        self.call_command("--no-input")

        self.user.refresh_from_db()

        self.assertEqual(
            first=self.user.email, second=f"{self.user.pk}@auth.testserver.net"
        )

    def test_starts_anonymization_when_user_confirms(self):
        """
        Test that the command starts anonymization when the user confirms

        :return:
        :rtype:
        """

        with (
            patch(
                "tnnt_templates.management.commands.tnnt_anonymize_emails.get_input"
            ) as mock_input,
            patch.object(Command, "_anonymize_user_emails") as mock_anonymize,
        ):
            mock_input.return_value = "yes"
            command = Command()
            command.handle(noinput=False)

        mock_input.assert_called_once_with(
            text="Are you sure you want to proceed? (yes/no) "
        )
        mock_anonymize.assert_called_once()

    def test_aborts_anonymization_when_user_declines(self):
        """
        Test that the command does not start anonymization when the user declines

        :return:
        :rtype:
        """

        with (
            patch(
                "tnnt_templates.management.commands.tnnt_anonymize_emails.get_input"
            ) as mock_input,
            patch.object(Command, "_anonymize_user_emails") as mock_anonymize,
        ):
            mock_input.return_value = "no"
            command = Command()
            command.handle(noinput=False)

        mock_input.assert_called_once_with(
            text="Are you sure you want to proceed? (yes/no) "
        )
        mock_anonymize.assert_not_called()

    def test_returns_user_input_when_valid_string_is_provided(self):
        """
        Test that the get_input function returns the user input when a valid string is provided.

        :return:
        :rtype:
        """

        with patch("builtins.input", return_value="yes"):
            result = get_input("Are you sure?")

            self.assertEqual(result, "yes")

    def test_handles_empty_input_gracefully(self):
        """
        Test that the get_input function handles empty input gracefully.

        :return:
        :rtype:
        """

        with patch("builtins.input", return_value=""):
            result = get_input("Enter something:")

            self.assertEqual(result, "")

    def test_handles_whitespace_only_input(self):
        """
        Test that the get_input function handles whitespace-only input correctly.

        :return:
        :rtype:
        """

        with patch("builtins.input", return_value="   "):
            result = get_input("Enter something:")

            self.assertEqual(result, "   ")

    def test_handles_special_characters_in_input(self):
        """
        Test that the get_input function handles special characters in input correctly.

        :return:
        :rtype:
        """

        with patch("builtins.input", return_value="!@#$%^&*()"):
            result = get_input("Enter special characters:")

            self.assertEqual(result, "!@#$%^&*()")



"""
TN-NT Templates app config
"""

# Django
from django.apps import AppConfig

from . import __version__


class TnntTemplatesConfig(AppConfig):
    """
    Template config
    """

    name = "tnnt_templates"
    label = "tnnt_templates"
    verbose_name = f"Terra Nanotech Alliance Auth Template Overrides v{__version__}"

    def ready(self) -> None:
        """
        App is ready

        :return:
        :rtype:
        """

        # AA Templates: Terra Nanotech
        import tnnt_templates.checks  # noqa: F401 # pylint: disable=unused-import, import-outside-toplevel



"""
Settings checks
"""

# Standard Library
from enum import Enum

# Django
from django.conf import settings
from django.core.checks import CheckMessage, register


class CheckLevel(Enum):
    """
    Check levels
    """

    DEBUG = 10
    INFO = 20
    WARNING = 30
    ERROR = 40
    CRITICAL = 50


@register()
def check_mandatory_settings(
    app_configs, **kwargs  # pylint: disable=unused-argument
) -> list[CheckMessage]:
    """
    Checks if the mandatory settings are set
    """

    errors: list[CheckMessage] = []

    mandatory_settings = ["TNNT_TEMPLATE_ENTITY_ID", "TNNT_TEMPLATE_ENTITY_NAME"]

    for setting in mandatory_settings:
        if not hasattr(settings, setting):
            errors.append(
                CheckMessage(
                    msg=f"Setting '{setting}' is missing",
                    hint="Please add this setting to your settings file",
                    id="tnnt_templates.E001",
                    level=CheckLevel.ERROR.value,
                ),
            )

    return errors



"""
Auth hooks for Terra Nanotech theme
"""

# Alliance Auth
from allianceauth import hooks
from allianceauth.theme.hooks import ThemeHook


class TerraNanotechThemeHook(ThemeHook):
    """
    Terra Nanotech Corp Auth Theme
    https://github.com/terra-nanotech/tn-nt-auth-templates/
    """

    def __init__(self):
        """
        Init
        """

        ThemeHook.__init__(
            self=self,
            name="Terra Nanotech",
            description="Terra Nanotech Corp Auth Theme",
            html_tags={"data-theme": "terra-nanotech"},
            css=[],
            js=[],
            css_template="tnnt_templates/includes/theme/css.html",
            js_template="tnnt_templates/includes/theme/js.html",
            header_padding="3.5em",
        )


@hooks.register("theme_hook")
def register_terra_nanotech_hook():
    """
    Registers the Terra Nanotech theme hook

    :return:
    :rtype:
    """

    return TerraNanotechThemeHook()



"""
Template filter for Terra Nanotech templates
"""

# Django
from django.apps import apps
from django.template.defaulttags import register


@register.filter
def startswith(haystack, needle):
    """
    Custom Django template filter to check if a string starts with a specified substring.

    This filter can be used in Django templates to evaluate whether the given string
    starts with the provided substring. If the condition is met,
    it returns `True`; otherwise, `False`.

    Usage:
        {% load tnnt_templates %}

        {% if string|starts_with:"substring" %}
            <!-- Render content if the string starts with the substring -->
        {% endif %}

    :param haystack: The string to be checked.
    :type haystack: str
    :param needle: The substring to check for at the start of `haystack`.
    :type needle: str
    :return: `True` if `haystack` starts with `needle`, otherwise `False`.
    :rtype: bool
    """

    return haystack.startswith(needle)


@register.simple_tag
def is_app_installed(app_name):
    """
    Custom Django template tag to check if a specific app is installed.

    This tag can be used in Django templates to determine whether a given app,
    identified by its name, is included in the `INSTALLED_APPS` setting. It returns
    `True` if the app is installed, otherwise `False`.

    Usage:
        {% load tnnt_templates %}

        {% is_app_installed "myapp" as is_myapp_installed %}
        {% if is_myapp_installed %}
            <!-- Render content if the app is installed -->
        {% endif %}

    :param app_name: The name of the app to check.
    :type app_name: str
    :return: `True` if the app is installed, otherwise `False`.
    :rtype: bool
    """

    return apps.is_installed(app_name)
